package cn.com.qmth.sdk.score;

import java.util.concurrent.atomic.AtomicBoolean;

import org.apache.commons.io.IOUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import cn.com.qmth.sdk.request.OuterDeleteScoreQueueTopReq;
import cn.com.qmth.sdk.request.OuterGetScoreDataReq;
import cn.com.qmth.sdk.request.OuterGetScoreQueueTopReq;
import cn.com.qmth.sdk.response.OuterGetScoreDataResp;
import cn.com.qmth.sdk.response.OuterGetScoreQueueTopResp;
import cn.com.qmth.sdk.util.HttpMethod;
import cn.com.qmth.sdk.util.JsonUtil;
import cn.com.qmth.sdk.util.OKHttpUtil;
import cn.com.qmth.sdk.util.QmthUtil;
import okhttp3.Response;

/**
 * 分数处理
 *
 * @author WANGWEI
 * @date 2018年11月23日
 * @Copyright (c) 2018-2020 WANGWEI [QQ:522080330] All Rights Reserved.
 */
public class ScoreHandler implements Runnable {

	private static final Logger LOG = LoggerFactory.getLogger(ScoreHandler.class);

	/**
	 * 有效数据处理
	 */
	private ValidScoreHandler validScoreHandler;

	/**
	 * 无效数据处理
	 */
	private InvalidScoreHandler invalidScoreHandler;

	private AtomicBoolean running;

	/**
	 * 构造函数
	 *
	 * @param validScoreHandler
	 * @param invalidScoreHandler
	 * @param running
	 */
	public ScoreHandler(ValidScoreHandler validScoreHandler,
			InvalidScoreHandler invalidScoreHandler, AtomicBoolean running) {
		super();
		this.validScoreHandler = validScoreHandler;
		this.invalidScoreHandler = invalidScoreHandler;
		this.running = running;
	}

	@Override
	public void run() {
		if (!running.compareAndSet(false, true)) {
			return;
		}
		int count = 0;
		while (true) {
			try {
				if (!execute()) {
					// 队列没有元素时线程退出
					break;
				}
			} catch (Exception e) {
				LOG.error("I am sorry", e);
				count++;
			}
			// 10次异常后线程退出
			if (10 < count) {
				break;
			}
		}

		if (!running.compareAndSet(true, false)) {
			throw new RuntimeException("status exception");
		}
	}

	private boolean execute() {
		// 查询队列
		String getElementUrl = QmthUtil.buildUrl("/api/exchange/outer/scoreQueue/getScoreQueueTop");
		OuterGetScoreQueueTopReq outerGetScoreQueueTopReq = new OuterGetScoreQueueTopReq();
		outerGetScoreQueueTopReq.setRootOrgId(QmthUtil.getRootOrgId());

		String outerGetScoreQueueTopRespJson = null;
		Response resp = null;
		try {
			resp = OKHttpUtil.call(HttpMethod.POST, getElementUrl, QmthUtil.getSecurityHeaders(),
					JsonUtil.toJson(outerGetScoreQueueTopReq));
			System.out.println(resp.code());
			System.out.println(resp.body().string());
			outerGetScoreQueueTopRespJson = resp.body().string();
		} catch (Exception e) {
			throw new RuntimeException(e);
		} finally {
			IOUtils.closeQuietly(resp);
		}

		OuterGetScoreQueueTopResp outerGetScoreQueueTopResp = JsonUtil
				.fromJson(outerGetScoreQueueTopRespJson, OuterGetScoreQueueTopResp.class);

		// 考试记录ID
		Long examRecordDataId = outerGetScoreQueueTopResp.getExamRecordDataId();
		if (null == examRecordDataId) {
			return false;
		}

		String getScoreDataUrl = QmthUtil.buildUrl("/api/exchange/outer/score/getScoreData");
		OuterGetScoreDataReq outerGetScoreDataReq = new OuterGetScoreDataReq();
		outerGetScoreDataReq.setExamRecordDataId(examRecordDataId);

		String outerGetScoreDataRespJson = null;
		try {
			resp = OKHttpUtil.call(HttpMethod.POST, getScoreDataUrl, QmthUtil.getSecurityHeaders(),
					JsonUtil.toJson(outerGetScoreDataReq));
			System.out.println(resp.code());
			System.out.println(resp.body().string());
			outerGetScoreDataRespJson = resp.body().string();
		} catch (Exception e) {
			throw new RuntimeException(e);
		} finally {
			IOUtils.closeQuietly(resp);
		}

		// 分数
		OuterGetScoreDataResp outerGetScoreDataResp = JsonUtil.fromJson(outerGetScoreDataRespJson,
				OuterGetScoreDataResp.class);

		Boolean isWarn = outerGetScoreDataResp.getIsWarn();
		Boolean isAudit = outerGetScoreDataResp.getIsAudit();
		Boolean isIllegality = outerGetScoreDataResp.getIsIllegality();

		// 正常状态1
		Boolean ok1 = (!isWarn) && (!isAudit) && (!isIllegality);
		// 正常状态2
		Boolean ok2 = (isWarn) && (isAudit) && (!isIllegality);

		if (ok1 || ok2) {
			validScoreHandler.handle(outerGetScoreDataResp);
		} else {
			invalidScoreHandler.handle(outerGetScoreDataResp);
		}

		String deleteQueueUrl = QmthUtil.buildUrl("/api/exchange/outer/scoreQueue/getScoreQueueTop");
		OuterDeleteScoreQueueTopReq outerDeleteScoreQueueTopReq = new OuterDeleteScoreQueueTopReq();
		outerGetScoreQueueTopReq.setRootOrgId(QmthUtil.getRootOrgId());
		OKHttpUtil.call(HttpMethod.POST, deleteQueueUrl, QmthUtil.getSecurityHeaders(),
				JsonUtil.toJson(outerDeleteScoreQueueTopReq));

		return true;
	}

}
