package cn.com.qmth.sdk.util;

import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.net.URLDecoder;

import org.apache.commons.lang3.StringUtils;

/**
 * 路径工具
 *
 * @author WANGWEI
 */
public class PathUtil
{

	/**
	 * 获取标准路径
	 *
	 * @author WANGWEI
	 * @param path
	 * @return
	 */
	public static String getCanonicalPath(String path)
	{
		path = path.replaceAll("\\+", "/").replaceAll("/+", "/").replaceAll("/\\./", "/");
		return StringUtils.replace(path, "/", File.separator);
	}

	/**
	 * 获取路径
	 *
	 * @author WANGWEI
	 * @param file
	 * @return
	 */
	public static String getCanonicalPath(File file)
	{
		try
		{
			return file.getCanonicalPath();
		}
		catch (IOException e)
		{
			throw new RuntimeException("Fail to get canonical path.", e);
		}

	}

	/**
	 * 获取当前路径
	 *
	 * @author WANGWEI
	 * @return
	 * @throws IOException
	 */
	public static String currentPath()
	{
		File directory = new File(". ");
		return getCanonicalPath(directory);
	}

	/**
	 * 获取资源路径
	 *
	 * @author WANGWEI
	 * @param resourceName
	 * @return
	 */
	public static String getResoucePath(String resourceName)
	{
		try
		{
			ClassLoader classLoader = PathUtil.class.getClassLoader();

			URL url = classLoader.getResource(resourceName);
			if (null != url)
			{
				String path = URLDecoder.decode(url.getPath(), "UTF-8");
				return path;
			}
			else
			{
				return null;
			}
		}
		catch (Exception e)
		{
			throw new RuntimeException(e);
		}
	}

	/**
	 * 获取windows盘符
	 *
	 * @author WANGWEI
	 * @param path
	 * @return
	 */
	public static String getDrive(String path)
	{
		if (path.matches("[a-zA-Z]:[\\\\/].*"))
		{
			return path.substring(0, 2);
		}
		else
		{
			throw new RuntimeException("Path is not a windows path.");
		}
	}

}
